#include <p18cxxx.h>
#include <xlcd.h>

#include "my_xlcd.h"

/********************************************************************
*       Function Name:  SetCGRamAddr                                *
*       Return Value:   void                                        *
*       Parameters:     CGaddr: character generator ram address     *
*       Description:    This routine sets the character generator   *
*                       address of the Hitachi HD44780 LCD          *
*                       controller. The user must check to see if   *
*                       the LCD controller is busy before calling   *
*                       this routine.                               *
********************************************************************/
void SetCGRamAddr(unsigned char CGaddr)
{
#ifdef BIT8                                     // 8-bit interface
        //TRIS_DATA_PORT = 0;                     // Make data port ouput
        my_data_port_eq(0);
        //DATA_PORT = CGaddr | 0b01000000;        // Write cmd and address to port
        my_set_data_port(CGaddr | 0b01000000);
        RW_PIN = 0;                             // Set control signals
        RS_PIN = 0;
        DelayFor18TCY();
        E_PIN = 1;                              // Clock cmd and address in
        DelayFor18TCY();
        E_PIN = 0;
        DelayFor18TCY();
        //TRIS_DATA_PORT = 0xff;                  // Make data port inputs
        my_tris_data_port_eq(0xff);
#else                                           // 4-bit interface
#ifdef UPPER                                    // Upper nibble interface
        //TRIS_DATA_PORT &= 0x0f;                 // Make nibble input
        my_tris_data_port_andeq(0x0f);
        //DATA_PORT &= 0x0f;                      // and write upper nibble
        my_data_port_andeq(0x0f);
        //DATA_PORT |= ((CGaddr | 0b01000000) & 0xf0);
        my_data_port_oreq( (CGaddr | 0b01000000) & 0xf0 );
#else                                           // Lower nibble interface
        //TRIS_DATA_PORT &= 0xf0;                 // Make nibble input
        my_tris_data_port_andeq(0xf0);
        //DATA_PORT &= 0xf0;   
        my_data_port_andeq(0xf0);                   // and write upper nibble
        //DATA_PORT |= (((CGaddr |0b01000000)>>4) & 0x0f);
        my_data_port_oreq(((CGaddr |0b01000000)>>4) & 0x0f);
#endif
        RW_PIN = 0;                             // Set control signals
        RS_PIN = 0;
        DelayFor18TCY();
        E_PIN = 1;                              // Clock cmd and address in
        DelayFor18TCY();
        E_PIN = 0;
#ifdef UPPER                                    // Upper nibble interface
        //DATA_PORT &= 0x0f;                      // Write lower nibble
        my_data_port_aneq(0x0f);
        //DATA_PORT |= ((CGaddr<<4)&0xf0);
        my_data_port_oreq((CGaddr<<4)&0xf0);
#else                                           // Lower nibble interface
        //DATA_PORT &= 0xf0;                      // Write lower nibble
        my_data_port_andeq(0xf0);
        //DATA_PORT |= (CGaddr&0x0f);
        my_data_port_oreq(CGaddr&0x0f);
#endif
        DelayFor18TCY();
        E_PIN = 1;                              // Clock cmd and address in
        DelayFor18TCY();
        E_PIN = 0;
#ifdef UPPER                                    // Upper nibble interface
        //TRIS_DATA_PORT |= 0xf0;                 // Make inputs
        my_tris_data_port_oreq(0xf0);
#else                                           // Lower nibble interface
        //TRIS_DATA_PORT |= 0x0f;                 // Make inputs
        my_tris_data_port_oreq(0x0f);
#endif
#endif
        return;
}

