
/** INCLUDES *******************************************************/
#include <stdlib.h>
#include <timers.h>
#include <delays.h>
#include <xlcd.h>
#include <string.h>

#include "sensors.h"
#include "p18f4550.h"

// INTOSC => 8 MHz
//#pragma config FOSC=INTOSC_XT

#pragma config PLLDIV=5
#pragma config CPUDIV=OSC1_PLL2
#pragma config USBDIV=2
#pragma config FOSC=HSPLL_HS

#pragma config FCMEN=OFF
#pragma config IESO=OFF
#pragma config VREGEN=ON
#pragma config CCP2MX=ON
#pragma config PBADEN=OFF

#pragma config PWRT=ON
#pragma config MCLRE=OFF
#pragma config BOR=ON		
#pragma config BORV=2

//#pragma config WDT=ON
#pragma config WDT=OFF
//#pragma config WDTPS=8192
	
#pragma config STVREN=ON
#pragma config LVP=OFF
#pragma config ICPRT= OFF
#pragma config XINST=OFF
#pragma config DEBUG=OFF
#pragma config WRTB=ON
#pragma config LPT1OSC=OFF

/** VARIABLES ******************************************************/
#pragma udata

static char hello[] = "Mr. Function...";
volatile unsigned int delay;

volatile unsigned long osc_count = 0ul;    
volatile unsigned long t0_delta = 0ul; 

extern unsigned int VALUE_DC;

//volatile int phase = P_ZERO;


/** PRIVATE PROTOTYPES *********************************************/

/* User defines these routines according to the oscillator frequency */

void DelayFor18TCY(void)
  {
    // DelayFor18TCY Delay for 18 cycles.
     
    Nop(); Nop(); Nop(); Nop(); Nop(); Nop();
    Nop(); Nop(); Nop(); Nop(); Nop(); Nop();
    Nop(); Nop(); Nop(); Nop(); Nop(); Nop();
    
    //Delay10TCYx(2);
  }
  
void DelayPORXLCD(void)
  {
    // DelayPORXLCD Delay for 15 ms.
    // clock = 48 MHz
    // 48 MHz * (1000*1000 Hz/1 MHz) * ((1 tick/second)/1 Hz) * (1 instruction)/(4 ticks) 
    // 48 MHz => 48*1000*1000/4 (instruction cycles)/(second)
    // 48 MHz => 12000000 instruction cycles/second
    // 8.3E-8 seconds / instruction
    // 0.000083 m sec = 1 instruction
    // 15 m sec = 181 k instruction
    Delay1KTCYx(181); //Delay in multiples of 1,000 instruction cycles.
  } 

void DelayXLCD(void) 
  {
    // DelayXLCD Delay for 5 ms.
    Delay1KTCYx(61);
  }

void clear_lcd(void)
  {
    // Clear display
    while(BusyXLCD());              // Wait if LCD busy
    WriteCmdXLCD(0x01);             // Clear display
  }
  
void print_lcd( char *s)
  {
    static char t[128];
   
    clear_lcd();
    
    strcpy(t,s);
    
    while( BusyXLCD() );
    putsXLCD( t );

  }

int linear(int x1, int y1, int x2, int y2, int x)
  {
    long u;
    long v;
    
    //return (y1+(y2-y1)*(x-x1)/(x2-x1));
    
    u=y2-y1;
    v=x-x1;    
    u=u*v;
    v=x2-x1;
    u=u/v;
    return (y1+u);
  }
  
int calculate_dc(void)
  {
    int v;
    
    if (VALUE_DC < 510u) 
      {
        v = linear(221,25,510,50,VALUE_DC);
      }
    else if (VALUE_DC < 645u)
      {
        v = linear(510,50,645,75,VALUE_DC);
      }
    else
      {
        v = linear(645,75,719,100,VALUE_DC);
      }
    
    if (v<0) return 0;
    if (v>100) return 100;
    return v;    
  }
    
void update_display(void)
  {
    unsigned long f;
    double d;
    static char buffer[32];
    static char Hz[] = " Hz ";
    static char pct[] = "% ";    
    static char buffer_dc[8];
    
    itoa(calculate_dc(),buffer_dc);
    //itoa(VALUE_DC,buffer_dc);
    
    // (osc_count) number of ticks in (t0_delta) seconds
    // (t0_delta ticks) * (32 instructions/1 t0_tick) * (4 cycles/1 instruction) * (1 sec/48*1000*1000 cycles)     
    //
    //seconds = t0_delta * 32 * 4 / 48000000;
    //seconds = t0_delta/375000;

    //f = (osc_count*375000ul)/t0_delta;
    d = ((double)375000.0)*((double)osc_count)/((double)t0_delta);
    f=d;
    
    ultoa(f,buffer);
    strcat(buffer,Hz);  
    strcat(buffer,buffer_dc);  
    strcat(buffer,pct);
    print_lcd(buffer);
    t0_delta = 0ul;
    osc_count = 0ul;
  }
    
void LowISR(void);
void HighISR(void);

#pragma code

#pragma interruptlow LowISR
void LowISR(void)
  {
  
  }


#pragma interruptlow HighISR
void HighISR(void)
  {	
    if (!((t0_delta == 0ul) && (osc_count == 0ul)))
      {
        INTCONbits.TMR0IF = 0;     // clear the interrupt
        PIR1bits.TMR1IF = 0;     // clear the interrupt                
        return;
      }
      
    //Check which interrupt flag caused the interrupt.
    if (INTCONbits.TMR0IF && PIR1bits.TMR1IF)
      {
        INTCONbits.TMR0IF = 0;     // clear the interrupt
        PIR1bits.TMR1IF = 0;     // clear the interrupt        
        
   		  t0_delta = ReadTimer0();
   		  osc_count = ReadTimer1();
   		  
   		  t0_delta += 65536ul;
   		  osc_count += 65536ul;   		  
   		  
   		  WriteTimer0(0);
   		  WriteTimer1(0);
      }  
  	else if (INTCONbits.TMR0IF)
  	  {
   	  	INTCONbits.TMR0IF = 0;     // clear the interrupt

   		  t0_delta = ReadTimer0();
   		  osc_count = ReadTimer1();
   		  
   		  t0_delta += 65536ul;
   		  
   		  WriteTimer0(0);
   		  WriteTimer1(0);
   		  
   		  // In this case, T0 overflowed.  So, we count the T1 value and compute the 
   		  // frequency as the T1 ticks per (seconds for t0 to overflow)
  	  }	
  	else if (PIR1bits.TMR1IF) 
      {
        PIR1bits.TMR1IF = 0;     // clear the interrupt
        
        
   		  t0_delta = ReadTimer0();
   		  osc_count = ReadTimer1();
   		  
   		  osc_count += 65536ul;
   		  
   		  WriteTimer0(0);
   		  WriteTimer1(0);
   		  
	      // In this case, T1 overflowed.  So, we count the T0 value and compute the 
   		  // frequency as the 65536 ticks per (seconds in t0 value)        
      }
  
  }




#pragma code lowVector=0x18
void LowVector(void){_asm goto LowISR _endasm}

#pragma code highVector=0x8
void HighVector(void){_asm goto HighISR _endasm}

#pragma code  // Return to default code section




void main(void)
  {   
//    unsigned int st_state = ST_START;
    
    ADCON1 |= 0x0F;                 // Default all pins to digital
    
    // ensure 8MHz clock
    //OSCCONbits.IRCF2 = 1;
    //OSCCONbits.IRCF1 = 1;
    //OSCCONbits.IRCF0 = 1;
    
    sensors_init();
    
    // The Timer0 clock interrupt forms our clock.
     OpenTimer0( 
        TIMER_INT_ON & 
        T0_SOURCE_INT &
        T0_16BIT &
        T0_PS_1_32
      ); 
      
    // seconds to overflow for T0_16BIT and T0_PS_1_32 is 0.17476267
    // http://www.camotruck.net/rollins/electronics/pic_timer.html

    // Timer1 counts the pulses from the external circuit.
    OpenTimer1( 
        TIMER_INT_ON & 
        T1_16BIT_RW &      // max count of 65535    
        T1_SOURCE_EXT &  
        T1_PS_1_1 &   // options are 1,2,4,8
        T1_OSC1EN_OFF &
        T1_SYNC_EXT_OFF 
      );

    OpenXLCD( EIGHT_BIT & LINE_5X7 );
    print_lcd(hello);          
    // INTCON=0;            //make sure interrupts are disable
    // INTCONbits.GIE=1;    //enable global interrupts
    // INTCONbits.PEIE=1;   //enable peripheral interrupts
    // INTCONbits.TMR0IE=1; //enable TMR0 overflow interrupt enable bit
    INTCON=0xe0;            //all 4 lines above combined
          
    while(1)
      {
          sensors_update();
          //  print_lcd(hello);          
          if ((osc_count > 0u) || (t0_delta > 0u))
            {
              update_display();
            }
      }
  }

